/*
 * pager.c	paginate a file for display
 ***************************************************************************
 *          Copyright (C) May 1993 Inge Cubitt and Peter Jones             *
 *                                                                         *
 * Conditions of licence are in accordance with terms laid out in the file *
 * called Licence distributed with this package.                           *
 *                                                                         *
 * Author: Peter L. Jones                                                  *
 ***************************************************************************
 */
#include <stdio.h>
#include <termios.h>
#define MY_BUFSZ (4096)

int pager(int tty,int file) {
    int LINES,WIDTH;
    struct winsize winsz;
    char reply[256], *buffer;
    int bpos;   /* offset from start of buffer to start of line */
    int esc;	/* flag: have we had an \033 this line? */
    int lines;	/* current screen line (sort of) */
    int lpos;	/* current screen column (hopefully) */
    int nr;	/* number of bytes read */
    int pos;    /* offset from start of line to current character */

    buffer=(char *)malloc(MY_BUFSZ);

    if (ioctl(tty,TIOCGWINSZ,&winsz) == 0) {
        LINES=winsz.ws_row;
        WIDTH=winsz.ws_col;
    } else {
        LINES=atoi(getenv("LINES"));
        if (LINES==0) {
            LINES=atoi(getenv("ROWS"));
            if (LINES==0) LINES=24;
        }
        WIDTH=atoi(getenv("WIDTH"));
        if (WIDTH==0) {
            WIDTH=atoi(getenv("COLUMNS"));
            if (WIDTH==0) WIDTH=80;
        }
    }
    WIDTH--;
    LINES--;

    esc = 0;
    lines = 0;
    nr = read(file,buffer,MY_BUFSZ);
    while(nr > 0) {
        bpos = 0;
        while(bpos < nr) {
            lpos = pos = 0;
            while( ((bpos+pos) < nr) && (esc == 0) && (lpos < WIDTH) && (buffer[bpos+pos] != '\n') ) {
                if (buffer[bpos+pos] == '\033') {
                    esc ++;
                } else if (buffer[bpos+pos] == '\t') {
                    while((lpos % 8) != 7) lpos++;
                }
                lpos++;
                pos++;
            }
            if (bpos < nr) {
                if (esc == 0) {
                    if (lpos < WIDTH) {
                        if (bpos+pos < nr) pos++; /* made conditional
						     PLJ 28/06/93 */
                        if (write(1,&buffer[bpos],pos) < pos) return(1);
                    } else {
                        if (write(1,&buffer[bpos],pos) < pos) return(1);
                        if (write(1,"\n",1) < 1) return(1);
                    }
                    lines++;
                    bpos += pos;
                    if (lines == LINES) {
                        if (write(1,"[C]ontinue or [s]top? ",22) < 22) return(1);
                        if (read(tty,reply,256) < 0) return(1);
                        switch(reply[0]) {
                            case 's':
                            case 'S':
                            case 'q':
                            case 'Q':
                                return(0);
                                break;
                            default:
                                if (strcmp("dumb",getenv("TERM")))
                                    if (write(1,"\033[A\r                      \r",27) < 27)
                                        return(1);
                                break;
                        }
                        lines = 0;
                    }
                } else {
                    if (write(1,&buffer[bpos],nr - bpos) < (nr - bpos)) return(1);
                    bpos = nr;
                }
            }
        }
        nr = read(file,buffer,MY_BUFSZ);
    }
    if (nr < 0) return(1);

    return(0);
}

main(argc,argv)
int argc;
char *argv[];
{
    int i,j,tty,file;
    char reply[11];

    switch(argc) {
    case 1:
        tty=open("/dev/tty",0);
        if (tty<0) {
            perror("/dev/tty");
            exit(1);
        }
        if (pager(tty,0)) {
            perror("STDIN");
            exit(1);
        }
        break;
    case 2:
        file=open(argv[1],0);
        if (file >= 0) {
            if (pager(0,file)) {
                perror(argv[1]);
                exit(1);
            }
        } else {
            printf("*** Can't open %s\n",argv[1]);
        }
        break;
    default:
        fprintf(stderr,"Usage: %s <file>\n   or: ... | %s\n",
            argv[0],argv[0]);
        exit(1);
    }
}
